@@ -6,6 +6,7 @@ from aspire.basis import Basis, FBBasisMixin
 from aspire.basis.basis_utils import real_sph_harmonic, sph_bessel, unique_coords_nd
 from aspire.utils import roll_dim, unroll_dim
 from aspire.utils.matlab_compat import m_flatten, m_reshape
+from aspire.volume import Volume
 
 logger = logging.getLogger(__name__)
 
@@ -25,7 +26,7 @@ class FBBasis3D(Basis, FBBasisMixin):
         :param size: The size of the vectors for which to define the basis.
             May be a 3-tuple or an integer, in which case a cubic basis is assumed.
             Currently only cubic images are supported.
-        :ell_max: The maximum order ell of the basis elements. If no input
+        :param ell_max: The maximum order ell of the basis elements. If no input
             (= None), it will be set to np.Inf and the basis includes all
             ell such that the resulting basis vectors are concentrated
             below the Nyquist frequency (default Inf).
@@ -140,9 +141,10 @@ class FBBasis3D(Basis, FBBasisMixin):
             * np.sqrt((self.nres / 2) ** 3)
         )
 
-    def evaluate(self, v):
+    def _evaluate(self, v):
         """
         Evaluate coefficients in standard 3D coordinate basis from those in FB basis
+
         :param v: A coefficient vector (or an array of coefficient vectors) to
             be evaluated. The first dimension must equal `self.count`.
         :return: The evaluation of the coefficient vector(s) `v` for this basis.
@@ -186,7 +188,7 @@ class FBBasis3D(Basis, FBBasisMixin):
 
         return x.T
 
-    def evaluate_t(self, v):
+    def _evaluate_t(self, v):
         """
         Evaluate coefficient in FB basis from those in standard 3D coordinate basis
 
@@ -197,7 +199,10 @@ class FBBasis3D(Basis, FBBasisMixin):
             equals `self.count` and whose remaining dimensions correspond
             to higher dimensions of `v`.
         """
-
+        # v may be a Volume object or a 7D array passed from Basis.mat_evaluate_t
+        # making this check important
+        if isinstance(v, Volume):
+            v = v.asnumpy()
         v = v.T
         x, sz_roll = unroll_dim(v, self.ndim + 1)
         x = m_reshape(
