@@ -6,7 +6,12 @@ from sklearn.neighbors import NearestNeighbors
 from tqdm import tqdm
 
 from aspire.basis import FSPCABasis
-from aspire.classification import BFSReddyChatterjiAverager2D, Class2D
+from aspire.classification import (
+    BFSReddyChatterjiAverager2D,
+    Class2D,
+    ClassSelector,
+    RandomClassSelector,
+)
 from aspire.classification.legacy_implementations import bispec_2drot_large, pca_y
 from aspire.numeric import ComplexPCA
 from aspire.utils.random import rand
@@ -19,7 +24,7 @@ class RIRClass2D(Class2D):
         self,
         src,
         pca_basis=None,
-        fspca_components=400,
+        fspca_components=None,
         alpha=1 / 3,
         sample_n=4000,
         bispectrum_components=300,
@@ -29,6 +34,9 @@ class RIRClass2D(Class2D):
         large_pca_implementation="legacy",
         nn_implementation="legacy",
         bispectrum_implementation="legacy",
+        selector=None,
+        num_procs=None,
+        batch_size=512,
         averager=None,
         dtype=None,
         seed=None,
@@ -47,18 +55,23 @@ class RIRClass2D(Class2D):
         for Viewing Direction Classification in Cryo-EM. (2014)
 
         :param src: Source instance.  Note it is possible to use one `source` for classification (ie CWF),
-        and a different `source` for stacking in the `averager`.
+            and a different `source` for stacking in the `averager`.
         :param pca_basis: Optional FSPCA Basis instance
-        :param fspca_components: Components (top eigvals) to keep from full FSCPA, default truncates to  400.
+        :param fspca_components: Optinally set number of components (top eigvals) to keep from full FSCPA.
+            Default value of None will infer from `pca_basis` when provided, otherwise defaults to 400.
         :param alpha: Amplitude Power Scale, default 1/3 (eq 20 from  RIIR paper).
         :param sample_n: Threshold for random sampling of bispectrum coefs. Default 4000,
-        high values such as 50000 reduce random sampling.
+            high values such as 50000 reduce random sampling.
         :param n_nbor: Number of nearest neighbors to compute.
         :param n_classes: Number of class averages to return.
         :param bispectrum_freq_cutoff: Truncate (zero) high k frequecies above (int) value, defaults off (None).
         :param large_pca_implementation: See `pca`.
         :param nn_implementation: See `nn_classification`.
         :param bispectrum_implementation: See `bispectrum`.
+        :param selector: A ClassSelector subclass. Defaults to RandomClassSelector.
+        :param num_procs: Number of processes to use.
+            `None` will attempt computing a suggestion based on machine resources.
+        :param batch_size: Chunk size (typically number of images) for batched methods.
         :param averager: An Averager2D subclass. Defaults to BFSReddyChatterjiAverager2D.
         :param dtype: Optional dtype, otherwise taken from src.
         :param seed: Optional RNG seed to be passed to random methods, (example Random NN).
@@ -72,41 +85,8 @@ class RIRClass2D(Class2D):
             seed=seed,
             dtype=dtype,
         )
-
-        # For now, only run with FSPCA basis
-        if pca_basis and not isinstance(pca_basis, FSPCABasis):
-            raise NotImplementedError(
-                "RIRClass2D has currently only been developed for pca_basis as a FSPCABasis."
-            )
-        self.pca_basis = pca_basis
-
-        # When a user provides a basis, the fspca_components arg is either
-        # redudant (same) or conflicting (different).
-        # So when a user provides fspca_components, we need to check it matches the basis' compoenents.
-        _provided_fspca_components = (
-            fspca_components is not self.__init__.__defaults__[1]
-        )
-        if pca_basis and _provided_fspca_components:
-            # Check the provided components match.
-            if pca_basis.components != fspca_components:
-                raise RuntimeError(
-                    f"`pca_basis` components {pca_basis.components} != {fspca_components} `fspca_components` provided by user."
-                )
-        elif pca_basis:  # fspca_components not specfied (default to taking from basis)
-            fspca_components = pca_basis.components
-        self.fspca_components = fspca_components
-
-        self.sample_n = sample_n
-        self.alpha = alpha
-        self.bispectrum_components = bispectrum_components
-        self.bispectrum_freq_cutoff = bispectrum_freq_cutoff
-        self.averager = averager
-
-        if self.src.n < self.bispectrum_components:
-            raise RuntimeError(
-                f"{self.src.n} Images too small for Bispectrum Components {self.bispectrum_components}."
-                "  Increase number of images or reduce components."
-            )
+        self.num_procs = num_procs
+        self.batch_size = int(batch_size)
 
         # Implementation Checks
         # # Do we have a sane Nearest Neighbor
@@ -148,6 +128,60 @@ class RIRClass2D(Class2D):
             )
         self._bispectrum = bispectrum_implementations[bispectrum_implementation]
 
+        # Setup class selection
+        if selector is None:
+            selector = RandomClassSelector(seed=self.seed)
+        elif not isinstance(selector, ClassSelector):
+            raise RuntimeError("`selector` must be subclass of `ClassSelector`")
+        self.selector = selector
+
+        # For now, only run with FSPCA basis
+        if pca_basis and not isinstance(pca_basis, FSPCABasis):
+            raise NotImplementedError(
+                "RIRClass2D has currently only been developed for pca_basis as a FSPCABasis."
+            )
+        self.pca_basis = pca_basis
+
+        # When a user provides a `pca_basis` and `fspca_components`
+        #  the arg is either redundant (same) or conflicting with `pca_basis`.
+        if pca_basis and fspca_components is not None:
+            # Check the provided components match. On match we can use this value.
+            if pca_basis.components != fspca_components:
+                raise RuntimeError(
+                    f"`pca_basis` components {pca_basis.components} != {fspca_components} `fspca_components` provided by user."
+                )
+        elif pca_basis:  # infer `fspca_components` from pca_basis components
+            fspca_components = pca_basis.components
+
+            # For small problems, such as unit tests, we also need to guard against
+            #   requesting more fspca_components than exist in the basis now,
+            # In the case RIRClass2d instantiates the pca_basis,
+            #   this will be checked then, in FSPCABasis.
+            pca_basis._check_components()
+        elif fspca_components is None:
+            # Default of 400 components was taken from legacy reearch and code.
+            fspca_components = 400
+
+        self.fspca_components = fspca_components
+        self.bispectrum_components = bispectrum_components
+        # Similarly, for small problems we need to check these counts.
+        if fspca_components < bispectrum_components:
+            raise RuntimeError(
+                f"fspca_components {fspca_components} < bispectrum components {bispectrum_components}."
+                "  Reduce bispectrum_components. Reasonable starting value is int(0.75*fspca_components)."
+            )
+
+        self.sample_n = sample_n
+        self.alpha = alpha
+        self.bispectrum_freq_cutoff = bispectrum_freq_cutoff
+        self.averager = averager
+
+        if self.src.n < self.bispectrum_components:
+            raise RuntimeError(
+                f"{self.src.n} Images too small for Bispectrum Components {self.bispectrum_components}."
+                "  Increase number of images or reduce components."
+            )
+
     def classify(self, diagnostics=False):
         """
         This is the high level method to perform the 2D images classification.
@@ -159,12 +193,10 @@ class RIRClass2D(Class2D):
         """
 
         # # Stage 1: Compute coef and reduce dimensionality.
-        # Memioze/batch this later when result is working
-        # Initial round of component truncation is before bispectrum.
-        #  default of 400 components was taken from legacy code.
-        # Instantiate a new compressed (truncated) basis.
         if self.pca_basis is None:
-            self.pca_basis = FSPCABasis(self.src, components=self.fspca_components)
+            self.pca_basis = FSPCABasis(
+                self.src, components=self.fspca_components, batch_size=self.batch_size
+            )
 
         # For convenience, assign the fb_basis used in the pca_basis.
         self.fb_basis = self.pca_basis.basis
@@ -173,7 +205,7 @@ class RIRClass2D(Class2D):
         #  we are certain our pca_basis has been constructed.
         if self.averager is None:
             self.averager = BFSReddyChatterjiAverager2D(
-                self.fb_basis, self.src, dtype=self.dtype
+                self.fb_basis, self.src, num_procs=self.num_procs, dtype=self.dtype
             )
 
         # Get the expanded coefs in the compressed FSPCA space.
@@ -203,10 +235,11 @@ class RIRClass2D(Class2D):
     def averages(self, classes, reflections, distances):
         # # Stage 3: Class Selection
         # This is an area open to active research.
-        # Currently we take a naive approach by selecting the
-        # first n_classes assuming they are quasi random.
         logger.info(f"Select {self.n_classes} Classes from Nearest Neighbors")
-        classes, reflections = self.select_classes(classes, reflections)
+        self.selection = selection = self.selector.select(
+            self.n_classes, classes, reflections, distances
+        )
+        classes, reflections = classes[selection], reflections[selection]
 
         # # Stage 4: Averager
         logger.info(
@@ -215,18 +248,6 @@ class RIRClass2D(Class2D):
 
         return self.averager.average(classes, reflections)
 
-    def select_classes(self, classes, reflections):
-        """
-        Select the `n_classes` to average from the (n_images) population of classes.
-        """
-        # Generate indices for random sample (can do something smarter, or build this out later).
-        # For testing/poc just take the first n_classes so it matches earlier plots for manual comparison
-        # If image stack is assumed to be reasonably random, this is a reasonable thing to do.
-        # Another reasonable thing would be to take a random selection over the whole dataset,
-        # in case the head of a dataset is too similar or has artifacts.
-        selection = np.arange(self.n_classes)
-        return classes[selection], reflections[selection]
-
     def pca(self, M):
         """
         Any PCA implementation here should return both
@@ -260,9 +281,9 @@ class RIRClass2D(Class2D):
         :param coef_b:
         :param coef_b_r:
         :returns:  Tuple of classes, refl, dists where
-        classes is an integer array of indices representing image ids,
-        refl is a bool array representing reflections (True is refl),
-        and distances is an array of distances as returned by NN implementation.
+            classes is an integer array of indices representing image ids,
+            refl is a bool array representing reflections (True is refl),
+            and distances is an array of distances as returned by NN implementation.
         """
         # _nn_classification is assigned during initialization.
         return self._nn_classification(coef_b, coef_b_r)
@@ -311,9 +332,7 @@ class RIRClass2D(Class2D):
 
         return classes, refl, distances
 
-        return indices
-
-    def _legacy_nn_classification(self, coeff_b, coeff_b_r, batch_size=2000):
+    def _legacy_nn_classification(self, coeff_b, coeff_b_r):
         """
         Perform nearest neighbor classification.
         """
@@ -333,13 +352,13 @@ class RIRClass2D(Class2D):
 
         concat_coeff = np.concatenate((coeff_b, coeff_b_r), axis=1)
 
-        num_batches = (n_im + batch_size - 1) // batch_size
+        num_batches = (n_im + self.batch_size - 1) // self.batch_size
 
         classes = np.zeros((n_im, n_nbor), dtype=int)
         distances = np.zeros((n_im, n_nbor), dtype=self.dtype)
         for i in range(num_batches):
-            start = i * batch_size
-            finish = min((i + 1) * batch_size, n_im)
+            start = i * self.batch_size
+            finish = min((i + 1) * self.batch_size, n_im)
             corr = np.real(
                 np.dot(np.conjugate(coeff_b[:, start:finish]).T, concat_coeff)
             )
