@@ -14,6 +14,7 @@ logger = logging.getLogger(__name__)
 def voltage_to_wavelength(voltage):
     """
     Convert from electron voltage to wavelength.
+
     :param voltage: float, The electron voltage in kV.
     :return: float, The electron wavelength in nm.
     """
@@ -23,6 +24,7 @@ def voltage_to_wavelength(voltage):
 def wavelength_to_voltage(wavelength):
     """
     Convert from electron voltage to wavelength.
+
     :param wavelength: float, The electron wavelength in nm.
     :return: float, The electron voltage in kV.
     """
@@ -35,8 +37,9 @@ def evaluate_src_filters_on_grid(src):
     """
     Given an ImageSource object, compute the source's unique filters
     at the filter_indices specified in its metadata.
+
     :return: an `src.L x src.L x len(src.filter_indices)`
-    array containing the evaluated filters at each gridpoint
+        array containing the evaluated filters at each gridpoint
     """
 
     grid2d = grid_2d(src.L, indexing="yx", dtype=src.dtype)
@@ -73,6 +76,7 @@ class Filter:
     def evaluate(self, omega):
         """
         Evaluate the filter at specified frequencies.
+
         :param omega: A vector of size n (for 1d filters), or an array of size 2-by-n, representing the spatial
             frequencies at which the filter is to be evaluated. These are normalized so that pi is equal to the Nyquist
             frequency.
@@ -108,6 +112,7 @@ class Filter:
     def scale(self, c=1):
         """
         Scale filter by a constant factor
+
         :param c: The scaling factor. For c < 1, it dilates the filter(s) in frequency, while for c > 1,
             it compresses (default 1).
         :return: A ScaledFilter object
@@ -139,6 +144,13 @@ class Filter:
     def dual(self):
         return DualFilter(self)
 
+    @property
+    def sign(self):
+        """
+        A Filter object to evaluate the signs of the underlying filter.
+        """
+        return LambdaFilter(self, np.sign)
+
 
 class DualFilter(Filter):
     """
@@ -249,6 +261,7 @@ class ScaledFilter(Filter):
     def __str__(self):
         """
         Show class name of ScaledFilter and related information
+
         :return: A string of class name and related information
         """
         return f"ScaledFilter (scales {self._filter} by {self._scale})"
@@ -258,6 +271,7 @@ class ArrayFilter(Filter):
     def __init__(self, xfer_fn_array):
         """
         A Filter corresponding to the filter with the specified transfer function.
+
         :param xfer_fn_array: The transfer function of the filter in the form of an array of one or two dimensions.
         """
         dim = xfer_fn_array.ndim
