@@ -8,9 +8,10 @@ import mrcfile
 import numpy as np
 
 from aspire.image import Image
-from aspire.operators import IdentityFilter
+from aspire.operators import CTFFilter, IdentityFilter
 from aspire.source.image import ImageSource
 from aspire.storage import StarFile
+from aspire.utils.relion_interop import RlnOpticsGroup
 
 logger = logging.getLogger(__name__)
 
@@ -44,7 +45,7 @@ class CoordinateSource(ImageSource, ABC):
     This also allows the CoordinateSource to be saved to an `.mrcs` stack.
     """
 
-    def __init__(self, files, particle_size, max_rows):
+    def __init__(self, files, particle_size, max_rows, B):
         mrc_paths, coord_paths = [f[0] for f in files], [f[1] for f in files]
         # the particle_size parameter is the *user-specified* argument
         # and is used in self._populate_particles
@@ -124,11 +125,10 @@ class CoordinateSource(ImageSource, ABC):
 
         ImageSource.__init__(self, L=L, n=n, dtype=dtype)
 
-        # Create filter indices for the source. These are required in order to
-        # pass through the filter eval code.
-        # Bypassing the filter_indices setter in ImageSource allows us
-        # create this source with absolutely *no* metadata.
-        # Otherwise, six default Relion columns are created w/defualt values
+        # CTF envelope decay factor
+        self.B = B
+        # set CTF metadata to defaults
+        # this can be updated with import_ctf()
         self.set_metadata("__filter_indices", np.zeros(self.n, dtype=int))
         self.unique_filters = [IdentityFilter()]
 
@@ -250,6 +250,171 @@ class CoordinateSource(ImageSource, ABC):
 
         return mrc_shapes
 
+    def import_ctf(self, ctf):
+        """
+        Given a RELION CTF STAR file, or list of CTF STAR files--one for each micrograph--, this function adds
+        corresponding CTFFilter objects to the `CoordinateSource`'s unique
+        filter list, and also populates the metadata.
+        """
+        self._ctf_cols = [
+            "_rlnVoltage",
+            "_rlnDefocusU",
+            "_rlnDefocusV",
+            "_rlnDefocusAngle",
+            "_rlnSphericalAberration",
+            "_rlnAmplitudeContrast",
+        ]
+
+        # attributes to be populated by the different CTF's
+        self._unique_filters = []
+        self._filter_indices = np.zeros(self.n, dtype=int)
+
+        # select method based on input type
+        if isinstance(ctf, str):
+            populate_ctf = self._populate_ctf_from_relion
+        elif isinstance(ctf, list):
+            populate_ctf = self._populate_ctf_from_list
+        else:
+            raise ValueError(
+                "Argument to import_ctf() must be a path or a list of paths"
+            )
+        # populate_ctf will update the arrays above
+        populate_ctf(ctf)
+
+        # populate filters and metadata
+        self.unique_filters = self._unique_filters
+        self.filter_indices = self._filter_indices
+
+    def _populate_ctf_from_relion(self, ctf_starfile):
+        """
+        Populates CTF filters and metadata based on a .star file with CTF parameters
+        in RELION format.
+        :param ctf_starfile: A RELION .star file containing CTF parameters for micrographs.
+        (Note: number of micrographs must match number of micrographs in CoordinateSource)
+        """
+        # RELION star files store CTF data in two separate blocks
+        star = StarFile(ctf_starfile)
+        optics = star["optics"]
+        micrographs = star["micrographs"]
+
+        # optics groups
+        optics_groups = [None]  # start indexing at 1
+        for _, row in optics.iterrows():
+            optics_groups.append(RlnOpticsGroup(row))
+
+        # micrographs
+        if not len(micrographs) == len(self.mrc_paths):
+            raise ValueError(
+                f"{ctf_starfile} has CTF information for {len(micrographs)}",
+                f" micrographs but this source has {len(self.mrc_paths)} micrographs.",
+            )
+
+        for mrc_idx, row in micrographs.iterrows():
+            # extract parameters not in the optics groups
+            params = {}
+            params["defocus_u"], params["defocus_v"], params["defocus_angle"] = (
+                float(row._rlnDefocusU),
+                float(row._rlnDefocusV),
+                float(row._rlnDefocusAngle),
+            )
+            # get parameters from corresponding optics group
+            optics_group = optics_groups[int(row._rlnOpticsGroup)]
+            # set the rest of the CTF parameters
+            (
+                params["voltage"],
+                params["cs"],
+                params["amplitude_contrast"],
+                params["pixel_size"],
+            ) = (
+                optics_group.voltage,
+                optics_group.cs,
+                optics_group.amplitude_contrast,
+                optics_group.pixel_size,
+            )
+            self._process_each_ctf(params, mrc_idx)
+
+    def _populate_ctf_from_list(
+        self,
+        ctf_files,
+    ):
+        """
+        Populates CTF filters and metadata based on a list of .star files containing CTF parameters..
+        :param ctf_files: A list of .star files containing CTF parameters for micrographs.
+        (Note: number of files must match number of micrographs in CoordinateSource)
+        """
+        if not len(ctf_files) == len(self.mrc_paths):
+            raise ValueError(
+                "Number of CTF STAR files must match number of micrographs."
+            )
+
+        for mrc_idx, ctf_file in enumerate(ctf_files):
+            params = self._read_ctf_star(ctf_file)
+            self._process_each_ctf(params, mrc_idx)
+
+    def _process_each_ctf(self, params, mrc_idx):
+        """
+        Given a unique set of CTF parameters, create a CTFFilter and populate source metadata.
+        :param params: Dictionary of CTF parameters.
+        :param mrc_idx: Index of the micrograph corresponding to these CTF parameters.
+        """
+        # find particle indices corresponding to this micrograph
+        indices = [
+            idx for idx, particle in enumerate(self.particles) if particle[0] == mrc_idx
+        ]
+        # add CTF filter to unique filters
+        self._unique_filters.append(
+            CTFFilter(
+                pixel_size=params["pixel_size"],
+                voltage=params["voltage"],
+                defocus_u=params["defocus_u"],
+                defocus_v=params["defocus_v"],
+                defocus_ang=params["defocus_angle"],
+                Cs=params["cs"],
+                alpha=params["amplitude_contrast"],
+                B=self.B,
+            )
+        )
+
+        # assign filter indices
+        self._filter_indices[indices] = mrc_idx
+        # populate CTF metadata
+        self.set_metadata(
+            self._ctf_cols,
+            np.array(
+                [
+                    [
+                        params["voltage"],
+                        params["defocus_u"],
+                        params["defocus_v"],
+                        params["defocus_angle"],
+                        params["cs"],
+                        params["amplitude_contrast"],
+                    ]
+                ]
+                * len(indices)
+            ),
+            indices,
+        )
+        # other ASPIRE metadata parameters
+        self.set_metadata("__mrc_filepath", self.mrc_paths[mrc_idx], indices)
+        self.set_metadata("__mrc_index", mrc_idx, indices)
+
+    def _read_ctf_star(self, ctf_file):
+        """
+        Reads a CTF STAR file generated by Relion or ASPIRE for a single
+        micrograph and returns a dictionary of CTF parameters and values.
+        """
+        df = StarFile(ctf_file).get_block_by_index(0)
+        return {
+            "defocus_u": float(df["_rlnDefocusU"][0]),
+            "defocus_v": float(df["_rlnDefocusV"][0]),
+            "defocus_angle": float(df["_rlnDefocusAngle"][0]),
+            "cs": float(df["_rlnSphericalAberration"][0]),
+            "amplitude_contrast": float(df["_rlnAmplitudeContrast"][0]),
+            "voltage": float(df["_rlnVoltage"][0]),
+            "pixel_size": float(df["_rlnDetectorPixelSize"][0]),
+        }
+
     @staticmethod
     def _crop_micrograph(data, coord):
         """
@@ -262,22 +427,25 @@ class CoordinateSource(ImageSource, ABC):
         start_x, start_y, size_x, size_y = coord
         return data[start_y : start_y + size_y, start_x : start_x + size_x]
 
-    def _images(self, start=0, num=np.inf, indices=None):
+    def _images(self, indices):
         """
         Given a range or selection of indices, returns an Image stack
-        of the particles specified. Note that the indices refer to the order
+        when accessed via the `ImageSource.images` property.
+        Note that the indices refer to the order
         of the particles loaded in this *specific* CoordinateSource. This may
         not correspond to the particles in the original source on disk, if some
         particles were excluded due to their box not fitting into the mrc
         dimensions. Thus, the exact particles returned are a function of the
         `particle_size`.
-        :param start: Starting index (default: 0)
-        :param num: number of images to return starting from `start` (default: numpy.inf)
-        :param indices: A numpy array of integer indices. If specified, supersedes
-        `start` and `num`.
+        :param indices: A 1-D NumPy array of integer indices.
+        :return: An `Image` object.
         """
-        if indices is None:
-            indices = np.arange(start, min(start + num, self.n))
+        # check for cached images first
+        if self._cached_im is not None:
+            logger.info("Loading images from cache")
+            return self.generation_pipeline.forward(
+                Image(self._cached_im[indices, :, :]), indices
+            )
 
         logger.info(f"Loading {len(indices)} images from micrographs")
 
@@ -322,8 +490,8 @@ class CoordinateSource(ImageSource, ABC):
                 if idx == mrc_index:
                     cropped = self._crop_micrograph(arr, next(coord))
                     im[i] = cropped
-
-        return Image(im)
+        # Finally, apply transforms to resulting Image
+        return self.generation_pipeline.forward(Image(im), indices)
 
     @staticmethod
     def _is_number(text):
@@ -345,6 +513,7 @@ class BoxesCoordinateSource(CoordinateSource):
         files,
         particle_size=None,
         max_rows=None,
+        B=0,
     ):
         """
         :param files: A list of tuples of the form (path_to_mrc, path_to_coord)
@@ -352,12 +521,7 @@ class BoxesCoordinateSource(CoordinateSource):
         :param max_rows: Maximum number of particles to read. (If `None`, will attempt to load all particles)
         """
         # instantiate super
-        CoordinateSource.__init__(
-            self,
-            files,
-            particle_size,
-            max_rows,
-        )
+        CoordinateSource.__init__(self, files, particle_size, max_rows, B)
 
     def _extract_box_size(self, box_file):
         with open(box_file, "r") as box:
@@ -460,12 +624,7 @@ class CentersCoordinateSource(CoordinateSource):
     Represents a data source consisting of micrographs and coordinate files specifying particle centers only. Files can be text (.coord) or STAR files.
     """
 
-    def __init__(
-        self,
-        files,
-        particle_size,
-        max_rows=None,
-    ):
+    def __init__(self, files, particle_size, max_rows=None, B=0):
         """
         :param files: A list of tuples of the form (path_to_mrc, path_to_coord)
         :particle_size: Desired size of cropped particles (mandatory)
@@ -473,12 +632,7 @@ class CentersCoordinateSource(CoordinateSource):
         attempt to load all particles)
         """
         # instantiate super
-        CoordinateSource.__init__(
-            self,
-            files,
-            particle_size,
-            max_rows,
-        )
+        CoordinateSource.__init__(self, files, particle_size, max_rows, B)
 
     def _validate_centers_file(self, coord_file):
         """
