@@ -1,6 +1,7 @@
 from .coor_trans import (  # isort:skip
     common_line_from_rots,
     crop_pad_2d,
+    crop_pad_3d,
     get_aligned_rotations,
     get_rots_mse,
     grid_1d,
@@ -10,11 +11,18 @@ from .coor_trans import (  # isort:skip
     uniform_random_angles,
 )
 from .misc import (  # isort:skip
+    all_pairs,
+    all_triplets,
     abs2,
     circ,
+    cyclic_rotations,
+    gaussian_1d,
     gaussian_2d,
+    gaussian_3d,
     get_full_version,
     inverse_r,
+    J_conjugate,
+    pairs_to_linear,
     powerset,
     sha256sum,
 )
@@ -42,5 +50,11 @@ from .matrix import (
     vol_to_vec,
     volmat_to_vecmat,
 )
+from .multiprocessing import (
+    mem_based_cpu_suggestion,
+    num_procs_suggestion,
+    physical_core_cpu_suggestion,
+    virtual_core_cpu_suggestion,
+)
 from .rotation import Rotation
 from .types import complex_type, real_type, utest_tolerance
